package org.linda.Client
import org.linda.LindaTypes._
import org.linda.Core._
import java.io._
import java.net.{InetAddress,ServerSocket,Socket,SocketException}
import java.util.Random

class Client {
  val port = 9998
  val ia = InetAddress.getByName("localhost")
  
  def out (input : Any) : Boolean =
    {
    try {
    	val socket = new Socket(ia, port)
    	val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    	val tuple = LindaTuple.createLindaTuple(input)
    	if (!tuple.isEmpty) {
    		outStream.writeObject(("out",LindaTuple.getSignature(tuple),tuple))
    		outStream.flush()
    		outStream.close()
    		true
    	}
    	else {
    		outStream.close()
    		false
    		}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        false
    	}
    }
   
  def out (input : List[List[LindaType]]) : Boolean = {
    if (input.isEmpty) false
    else
      {
    	input.foreach (tuple => this.out(tuple))
    	true
    }
   } 

  def out (input : Tuple2[Boolean,List[List[LindaType]]]) : Boolean = {
    val (bool, tupleList) = input
    if (bool) 
      {
    	tupleList.foreach (tuple => this.out(tuple))
    	true
      }
    else
      false
   } 
 
  def in (input : Any) : List[LindaType] = {
   try {
    val socket = new Socket(ia, port) //створення нового сокету
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream())) //створення нового вихідного потоку
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream())) //створення нового вхідного потоку
    val antiTuple = LindaTuple.createLindaTuple(input) //створення кортежу Linda
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("in","",antiTuple)) //передача запиту на сервер
    	outStream.flush()
    	val tuple = inStream.readObject().asInstanceOf[List[LindaType]] //читання об'єкту від сервера
    	outStream.close() //закриття потоків
    	inStream.close()
    	tuple //повернення знайденого кортежу процесу
    	}
    else {
    	outStream.close()
    	inStream.close()
    	List ()
     	}
    }
   catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def rd (input : Any) : List[LindaType] = {
   try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("rd","",antiTuple))
    	outStream.flush()
    	val tuple = inStream.readObject().asInstanceOf[List[LindaType]]
        outStream.close()
        inStream.close()
        tuple
    	}
    else {
    	outStream.close()
        inStream.close()
        List()
    }
    	
    }
   catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }

  def inp (input : Any) = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("inp","",antiTuple))
    	outStream.flush()
    	val (found,tuple) = inStream.readObject().asInstanceOf[Tuple2[Boolean,List[LindaType]]]
        outStream.close()
        inStream.close()
        (found,tuple)
    }
    else {
    	outStream.close()
        inStream.close()
        (false,List())
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def rdp (input : Any)  = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("rdp","",antiTuple))
    	outStream.flush()
    	val (found,tuple) = inStream.readObject().asInstanceOf[Tuple2[Boolean,List[LindaType]]]
        outStream.close()
        inStream.close()
        (found,tuple)
    	}
    else {
    	outStream.close()
        inStream.close()
        (false,List())
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        (false,List())
   		}
    }
  
  def collect (input : Any) : List[List[LindaType]] = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("collect","",antiTuple))
    	outStream.flush()
    	val tupleList = inStream.readObject().asInstanceOf[List[List[LindaType]]]
        outStream.close()
        inStream.close()
        tupleList
    	}
    else {
       outStream.close()
       inStream.close()
       List()
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def copycollect (input : Any) : List[List[LindaType]] = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("copycollect","",antiTuple))
    	outStream.flush()
    	val tupleList = inStream.readObject().asInstanceOf[List[List[LindaType]]]
        outStream.close()
        inStream.close()
        tupleList
    	}
    else {
    	outStream.close()
        inStream.close()
        List()
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def query (length : Int, input : Any) : List[List[LindaType]] = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("query",length,antiTuple))
    	outStream.flush()
    	val tupleList = inStream.readObject().asInstanceOf[List[List[LindaType]]]
        outStream.close()
        inStream.close()
        tupleList
    	}
    else {
    	outStream.close()
        inStream.close()
        List()
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def queryp (length : Int ,input : Any)  = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    outStream.writeObject(("queryp",length,antiTuple))
    outStream.flush()
    val (found,tuple) = inStream.readObject().asInstanceOf[Tuple2[Boolean,List[List[LindaType]]]]
    outStream.close()
    inStream.close()
    (found,tuple)
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        (false,List())
   		}
    }
 
  def copyquery (length : Int, input : Any) : List[List[LindaType]] = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("copyquery",length,antiTuple))
    	outStream.flush()
    	val tupleList = inStream.readObject().asInstanceOf[List[List[LindaType]]]
        outStream.close()
        inStream.close()
        tupleList
    	}
    else {
    	outStream.close()
        inStream.close()
        List()
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        List()
   		}
    }
  
  def copyqueryp (amount : Int ,input : Any)  = {
    try {
    val socket = new Socket(ia, port)
    val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
    val inStream = new ObjectInputStream (new DataInputStream(socket.getInputStream()))
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	outStream.writeObject(("copyqueryp",amount,antiTuple))
    	outStream.flush()
    	val (found,tuple) = inStream.readObject().asInstanceOf[Tuple2[Boolean,List[List[LindaType]]]]
        outStream.close()
        inStream.close()
        (found,tuple)
    	}
    else {
    	outStream.close()
        inStream.close()
        (false,List())
    	}
    }
    catch { 
       case e: IOException =>
        System.err.println("Could not connect to port: "+port+".")
        System.exit(-1)
        (false,List())
   		}
    }
}

