package org.linda.Core
import org.linda.Database.Database
import org.linda.Core._
import org.linda.LindaTypes.LindaType

class LocalSpace {
  val db = new Database
  
  def out (input : Tuple2[Boolean,List[List[LindaType]]]) : Boolean = {
    //println(input)
    val (bool, tupleList) = input
    if (bool) 
      {
    	db.out(tupleList)
        db.notifyAll()
    	true
      }
    else
      false
   } 
   
  def out (input : Any) :  Boolean = {
      val tuple = LindaTuple.createLindaTuple(input)
      if (!tuple.isEmpty) {
    	  db.synchronized {
    	  db.out (LindaTuple.getSignature(tuple),tuple)
    	  db.notifyAll()
    	  true
    	  }
      }
      else 
    	  false
    } 

   def out (input : List[List[LindaType]]) : Boolean = {
    //println(input)
    if (input.isEmpty) false
    else
      {
    	db.out(input)
    	db.notifyAll()
    	true
    }
   } 
   
  def in (input : Any) : List[LindaType] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	db.synchronized {
    		while (db.find(antiTuple).isEmpty)
    		wait ()
    		db.in(antiTuple)
    	}
    }
    else 
      List()
  }
  
  def rd (input : Any) : List[LindaType] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	db.synchronized {
    		while (db.find(antiTuple).isEmpty)
    			wait ()
    	db.rd(antiTuple)
    	}
    }
    else
      List()
  }
  
  def inp (input : Any) = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	if (!(db.find(antiTuple).isEmpty))
    		(true,db.in(antiTuple))
    	else
    		(false, List())
    	}
    else
      (false, List())
    }
  
  def rdp (input : Any)  = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	if (!(db.find(antiTuple).isEmpty))
    		(true,db.rd(antiTuple))
    	else
    		(false, List())
    }
    else 
      (false, List())
  }
  
  def collect (input : Any) : List[List[LindaType]] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {	
    	if (!(db.find(antiTuple).isEmpty))
    		db.collect(antiTuple)
    	else
    		List()
    }
    else 
      List()
   }
  
  def copycollect (input : Any) : List[List[LindaType]] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	if (!(db.find(antiTuple).isEmpty))
    		db.copycollect(antiTuple)
    	else
    		List()
    	}
    else 
      List()
  }
    
  def query (length : Int, input : Any) : List[List[LindaType]] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	db.synchronized 
    		{
    		while (db.find(antiTuple).length < length)
    			db.wait()
    		while (db.getLength(antiTuple) < length)
    			db.wait
    		db.query(length,antiTuple)
    		}
      }
    else 
    	List()
    }
  
  def queryp (length : Int ,input : Any)  = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	if (db.getLength(antiTuple) < length)
    		(false,List())
    	else if (db.find(antiTuple).length < length)
    			(false,List())
    		else
    			(true,db.queryp(length,antiTuple))
     }
    else (false, List())
  }
  
  def copyquery (length : Int, input : Any) : List[List[LindaType]] = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	db.synchronized 
    		{
    		while (db.find(antiTuple).length < length)
    			db.wait()
    		while (db.getLength(antiTuple) < length)
    			db.wait
    		db.copyquery(length,antiTuple)
    		}
      }
    else 
      List()
  }
  def copyqueryp (length : Int ,input : Any)  = {
    val antiTuple = LindaTuple.createLindaTuple(input)
    if (!antiTuple.isEmpty) {
    	if (db.getLength(antiTuple) < length)
    		(false,List())
    	else if (db.find(antiTuple).length < length)
    			(false,List())
    		 else
    			 (true,db.copyquery(length,antiTuple))
    	}
    else 
      (false,List())
    }
}
