package org.linda.Server
import org.linda.Database._
import org.linda.Core._
import org.linda.LindaTypes.LindaType
import java.io._
import java.net.{InetAddress,ServerSocket,Socket,SocketException}
import java.util.Random


class LindaServer {
  private val db = new Database
  private val port = 9998
  
  def start ()
  {
    try {
      println("server has started")
      //val t = Thread.currentThread()
      //println(t.getPriority)
      val listener = new ServerSocket(port)
      //listener.
      while (true)
          new ServerThread(listener.accept(),db).start()
      //new ServerThread(listener.accept).start()
      
      listener.close()
    }
    catch {
      case e: IOException =>
        System.err.println("Could not listen on port: "+port+".")
        System.exit(-1)
    }
  }


}

case class ServerThread(socket: Socket, db : Database) extends Thread("ServerThread") {

  
  override def run(): Unit = {
    try {
      val outStream = new ObjectOutputStream(new DataOutputStream(socket.getOutputStream()))
      val inStream = new ObjectInputStream(new DataInputStream(socket.getInputStream()))
      val (operation,signature,input) = inStream.readObject()
      val tuple = input.asInstanceOf[List[LindaType]]
      db.synchronized {
      operation match  {
        case "out" => db.out(signature.asInstanceOf[String],tuple); db.notifyAll()
        case "in" =>  {while (db.find(tuple).isEmpty)
            //println("no such element")
            //println(db.contains(signature.asInstanceOf[String]))
            //println(signature)
            //println(tuple)
            //println(db.find(tuple.asInstanceOf[List[LindaType]]))
            db.wait()
          outStream.writeObject(db.in(tuple))
        }
        case "rd" => {while (db.find(tuple).isEmpty)
                        db.wait()
                      outStream.writeObject(db.rd(tuple))
        			 }
        case "inp" => {if (!(db.find(tuple).isEmpty))
                      	outStream.writeObject((true,db.in(tuple)))
                       else
                        outStream.writeObject((false,List()))
        			 }
        case "rdp" => {if (!(db.find(tuple).isEmpty))
                      	outStream.writeObject((true,db.rd(tuple)))
                       else
                        outStream.writeObject((false,List()))
        			 }
        case "collect" => {if (!(db.find(tuple).isEmpty))
        					outStream.writeObject(db.collect(tuple))
        				   else
        					outStream.writeObject((List()))
        			 }
        case "copycollect" => {if (!(db.find(tuple).isEmpty))
        						outStream.writeObject(db.copycollect(tuple))
        					   else
        						outStream.writeObject((List()))
        			 }
        case "query" => {while (db.find(tuple).length < signature.asInstanceOf[Int])
        					db.wait()
                         while (db.getLength(tuple) < signature.asInstanceOf[Int])
                           db.wait
                         outStream.writeObject(db.query(signature.asInstanceOf[Int],tuple))
        			 }
        case "queryp" => {if (db.getLength(tuple) < signature.asInstanceOf[Int])
                            outStream.writeObject((false,List()))
                          else if (db.find(tuple).length < signature.asInstanceOf[Int])
                        	  	outStream.writeObject((false,List()))
                        	  else
                        		outStream.writeObject((true,db.queryp(signature.asInstanceOf[Int],tuple)))
        			 }
        case "copyquery" => {while (db.find(tuple).length < signature.asInstanceOf[Int])
        						db.wait()
        					 while (db.getLength(tuple) < signature.asInstanceOf[Int])
        						db.wait
        					 outStream.writeObject(db.copyquery(signature.asInstanceOf[Int],tuple))
        					 outStream.flush()
        			 }
        case "copyqueryp" => {if (db.getLength(tuple) < signature.asInstanceOf[Int])
        						outStream.writeObject((false,List()))
        					  else if (db.find(tuple).length < signature.asInstanceOf[Int])
                        	  		outStream.writeObject((false,List()))
                        	       else
                        		    outStream.writeObject((true,db.copyquery(signature.asInstanceOf[Int],tuple)))
        			 }
        case _ => println("nothing matched")
      }
      }
      //val test=inStream.readObject().asInstanceOf[String]
      //println("in the thread") 
      outStream.close();
      inStream.close();
      socket.close()
    }
    catch {
      case e: SocketException =>
        () // avoid stack trace when stopping a client with Ctrl-C
      case e: IOException =>
        e.printStackTrace();
    }
  }

}